/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: NDPCAL_DO_PRATO_PLA.SQL

Descrio:
Retorna o NDPCal do Prato.

P_CDFILIAL   - Cdigo da Filial. Pode ser passado vazio para consultar
               o NDPCal da receita padro.
P_CDSERVFILI - Cdigo do Servico. Pode ser passado vazio para consultar
               o NDPCal da receita padro ou por filial.
P_CDFILAUXPRAT - Cdigo da filial do prato.
P_CDPRATO    - Cdigo do Prato.

****************************************************************
             CORREO DO CLCULO DO NDPCAL%
****************************************************************
A frmula usada para clculo do NDPCal% :

NDPCAL% = NPU * Sum(Protenas) * 4
          ------------------------
                    VCT
onde:

NPU = Somatrio das protenas dos componentes da receita multiplicados
      pelo Fator de Clculo de NDPCAL, informado na tela de parametrizao
      por grupo de produtos, sendo 0,7 para alimentos de origem animal;
      0,6 para leguminosas e 0,5 para cereais, frutas e outros vegetais.
Sum(Protenas) = Somatrio das protenas dos componentes da receita
VCT = "Valor Calrico Total" = Soma(carboidratos) x 4 +
                               Soma(lipdeos)    x 9  +
                               Soma(protenas)   x 4  +
                               Soma(lcoois)     x 7
****************************************************************
*/

IF EXISTS (SELECT *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'NDPCAL_PRATO' AND
                 TYPE = 'FN'             AND
                 U.UID = O.UID           AND
                 U.NAME = USER
          )
   DROP FUNCTION NDPCAL_PRATO
GO

CREATE FUNCTION NDPCAL_PRATO(
    @P_CDFILIAL     VARCHAR(4),
    @P_CDSERVFILI   VARCHAR(5),
    @P_CDFILAUXPRAT VARCHAR(4),
    @P_CDPRATO      VARCHAR(10),
    @P_CDFILCENTPME VARCHAR(4) = NULL,
    @P_CDTIPOCARD   VARCHAR(10) = NULL,
    @P_MERENDA      VARCHAR(1) = 'N') RETURNS NUMERIC(15,6)
AS
BEGIN 
   DECLARE @V_NDPCAL         NUMERIC(15,6);
   DECLARE @V_CALORIAS       NUMERIC(15,6);
   DECLARE @V_CUR_PROTEINA   NUMERIC(15,6);
   DECLARE @V_CUR_CALORIA_CARBOIDRATO NUMERIC(15,6);
   DECLARE @V_CUR_CALORIA_LIPIDEO NUMERIC(15,6);
   DECLARE @V_CUR_CALORIA_PROTEINA NUMERIC(15,6);
   DECLARE @V_CUR_CALORIA_ALCOOL NUMERIC(15,6);
   DECLARE @V_NPU NUMERIC (15,6);


   /* ----------------- RETORNA O TOTAL DE CALORIAS DO PRATO ---------------- */

   DECLARE CUR_CALORIA CURSOR FOR
     SELECT
        SUM(NP.QTNUTRPROD * QTPRODUTO) * 4 AS PROTEINA,
        SUM(NC.QTNUTRPROD * QTPRODUTO) * 4 AS CARBOIDRATO,
        SUM(NL.QTNUTRPROD * QTPRODUTO) * 9 AS LIPIDEO,
        SUM(NA.QTNUTRPROD * QTPRODUTO) * 7 AS ALCOOL
      FROM (
         /*receita por servio*/ 
         SELECT RS.CDPRODUTO, P.CDCARBOHIDRA, P.CDPROTEINA, P.CDLIPIDIOS, P.CDALCOOL,
                (10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRSE) AS QTPRODUTO
           FROM (SELECT MAX(CDCARBOHIDRA) CDCARBOHIDRA, MAX(CDPROTEINA) CDPROTEINA,
                        MAX(CDLIPIDIOS) CDLIPIDIOS, MAX(CDALCOOL) CDALCOOL FROM PARAMGERAL) P,
                PRODUTO PR, PRODFILI, RECESERV RS
          WHERE (RS.CDFILIAL        = @P_CDFILIAL)                        
            AND (RS.CDSERVFILI      = @P_CDSERVFILI)                      
            AND (RS.CDFILAUXPRAT    = @P_CDFILAUXPRAT)                    
            AND (RS.CDPRATO         = @P_CDPRATO)                         
            AND (RS.CDPRODUTO       = PR.CDPRODUTO)                       
            AND (PRODFILI.CDFILIAL  = @P_CDFILIAL)                      
            AND (PRODFILI.CDPRODUTO = PR.CDPRODUTO)
         UNION ALL
         /*receita por filial*/  
         SELECT RF.CDPRODUTO, P.CDCARBOHIDRA, P.CDPROTEINA, P.CDLIPIDIOS, P.CDALCOOL,
               (10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRLO) AS QTPRODUTO
           FROM (SELECT MAX(CDCARBOHIDRA) CDCARBOHIDRA, MAX(CDPROTEINA) CDPROTEINA,
                        MAX(CDLIPIDIOS) CDLIPIDIOS, MAX(CDALCOOL) CDALCOOL FROM PARAMGERAL) P,
                 PRODUTO PR, PRODFILI, RECEFILI RF
          WHERE (RF.CDFILIAL        = @P_CDFILIAL)                         
            AND (RF.CDFILAUXPRAT    = @P_CDFILAUXPRAT)                     
            AND (RF.CDPRATO         = @P_CDPRATO)                          
            AND (RF.CDPRODUTO       = PR.CDPRODUTO)                        
            AND (PRODFILI.CDFILIAL  = @P_CDFILIAL)                         
            AND (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                        
            AND NOT EXISTS (SELECT CDPRATO
                              FROM RECESERV
                             WHERE (CDFILAUXPRAT = RF.CDFILAUXPRAT) 
                               AND (CDPRATO      = RF.CDPRATO)      
                               AND (CDFILIAL     = RF.CDFILIAL)     
                               AND (CDSERVFILI   = @P_CDSERVFILI))
         UNION ALL
         /*receita por tipo de cardpio*/ 
         SELECT RT.CDPRODUTO, P.CDCARBOHIDRA, P.CDPROTEINA, P.CDLIPIDIOS, P.CDALCOOL,
               (10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRPRTP) AS QTPRODUTO
           FROM (SELECT MAX(CDCARBOHIDRA) CDCARBOHIDRA, MAX(CDPROTEINA) CDPROTEINA,
                        MAX(CDLIPIDIOS) CDLIPIDIOS, MAX(CDALCOOL) CDALCOOL FROM PARAMGERAL) P,
                PRODUTO PR, PRODFILI, RECETPCARD RT
          WHERE (RT.CDFILAUXPRAT    = @P_CDFILAUXPRAT)                       
            AND (RT.CDPRATO         = @P_CDPRATO)                            
            AND (RT.CDPRODUTO       = PR.CDPRODUTO)                          
            AND (PRODFILI.CDFILIAL  = @P_CDFILIAL)                           
            AND (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                          
            AND (@P_MERENDA = 'S')
            AND NOT EXISTS (SELECT CDPRATO
                              FROM RECEFILI
                             WHERE (CDFILAUXPRAT = RT.CDFILAUXPRAT) 
                               AND (CDPRATO      = RT.CDPRATO)      
                               AND (CDFILIAL     = @P_CDFILIAL)) 
            AND NOT EXISTS (SELECT CDPRATO
                              FROM RECESERV
                             WHERE (CDFILAUXPRAT = RT.CDFILAUXPRAT)  
                               AND (CDPRATO      = RT.CDPRATO)       
                               AND (CDFILIAL     = @P_CDFILIAL)      
                               AND (CDSERVFILI   = @P_CDSERVFILI))
         UNION ALL
         /*receita padro*/
         SELECT RP.CDPRODUTO, P.CDCARBOHIDRA, P.CDPROTEINA, P.CDLIPIDIOS, P.CDALCOOL,
               (10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRPR) AS QTPRODUTO
           FROM (SELECT MAX(CDCARBOHIDRA) CDCARBOHIDRA, MAX(CDPROTEINA) CDPROTEINA,
                        MAX(CDLIPIDIOS) CDLIPIDIOS, MAX(CDALCOOL) CDALCOOL FROM PARAMGERAL) P,
                PRODUTO PR, PRODFILI, RECEPADR RP
          WHERE (RP.CDFILAUXPRAT    = @P_CDFILAUXPRAT)                       
            AND (RP.CDPRATO         = @P_CDPRATO)                            
            AND (RP.CDPRODUTO       = PR.CDPRODUTO)                          
            AND (PRODFILI.CDFILIAL  = @P_CDFILIAL)                           
            AND (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                          
            AND NOT EXISTS (SELECT CDPRATO
                              FROM RECETPCARD
                             WHERE (CDFILAUXPRAT = RP.CDFILAUXPRAT) 
                               AND (CDPRATO      = RP.CDPRATO)      
                               AND (CDFILCENTPME = @P_CDFILCENTPME) 
                               AND (CDTIPOCARD   = @P_CDTIPOCARD
                               AND (@P_MERENDA = 'S')))   
            AND NOT EXISTS (SELECT CDPRATO
                              FROM RECEFILI
                             WHERE (CDFILAUXPRAT = RP.CDFILAUXPRAT) 
                               AND (CDPRATO      = RP.CDPRATO)      
                               AND (CDFILIAL     = @P_CDFILIAL))    
            AND NOT EXISTS (SELECT CDPRATO
                              FROM RECESERV
                             WHERE (CDFILAUXPRAT = RP.CDFILAUXPRAT)
                               AND (CDPRATO      = RP.CDPRATO)     
                               AND (CDFILIAL     = @P_CDFILIAL)    
                               AND (CDSERVFILI   = @P_CDSERVFILI))
         ) R
         LEFT JOIN NUTRPROD NP ON
         NP.CDPRODUTO  = R.CDPRODUTO AND
         NP.CDNUTRIENTE = R.CDPROTEINA
         LEFT JOIN NUTRPROD NC ON
         NC.CDPRODUTO  = R.CDPRODUTO AND
         NC.CDNUTRIENTE = R.CDCARBOHIDRA
         LEFT JOIN NUTRPROD NL ON
         NL.CDPRODUTO  = R.CDPRODUTO AND
         NL.CDNUTRIENTE = R.CDLIPIDIOS
         LEFT JOIN NUTRPROD NA ON
         NA.CDPRODUTO = R.CDPRODUTO AND
         NA.CDNUTRIENTE = R.CDALCOOL;


   OPEN CUR_CALORIA;
   FETCH NEXT FROM CUR_CALORIA INTO @V_CUR_CALORIA_PROTEINA, @V_CUR_CALORIA_CARBOIDRATO, @V_CUR_CALORIA_LIPIDEO, @V_CUR_CALORIA_ALCOOL;

   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      SET @V_CALORIAS = ISNULL(@V_CUR_CALORIA_CARBOIDRATO,0) + ISNULL(@V_CUR_CALORIA_LIPIDEO,0) + ISNULL(@V_CUR_CALORIA_PROTEINA,0) + ISNULL(@V_CUR_CALORIA_ALCOOL,0);
      FETCH NEXT FROM CUR_CALORIA INTO @V_CUR_CALORIA_PROTEINA, @V_CUR_CALORIA_CARBOIDRATO, @V_CUR_CALORIA_LIPIDEO, @V_CUR_CALORIA_ALCOOL;
   END;

   DEALLOCATE CUR_CALORIA;

   /* ---------------- RETORNA O TOTAL DE PROTEINAS DO PRATO ---------------- */
   DECLARE CUR_PROTEINA CURSOR FOR
      SELECT SUM(PROTEINA) PROTEINA, SUM(NPU) NPU 
        FROM (/*receita por servio*/
              SELECT SUM(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * RS.QTCALCNUTRSE) AS PROTEINA,
                     SUM(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * RS.QTCALCNUTRSE * NC.VRFATCALNDPC) AS NPU
                FROM RECESERV RS, NUTRPROD NP, PRODUTO PR, NDPCAL NC, PRODFILI
               WHERE (RS.CDFILIAL        = @P_CDFILIAL)                           
                 AND (RS.CDSERVFILI      = @P_CDSERVFILI)                         
                 AND (RS.CDFILAUXPRAT    = @P_CDFILAUXPRAT)                       
                 AND (RS.CDPRATO         = @P_CDPRATO)                            
                 AND (RS.CDPRODUTO       = NP.CDPRODUTO)                          
                 AND (RS.CDPRODUTO       = PR.CDPRODUTO)                          
                 AND (PRODFILI.CDFILIAL  = @P_CDFILIAL)                           
                 AND (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                          
                 AND (NP.CDNUTRIENTE  = (SELECT MAX(CDPROTEINA) FROM PARAMGERAL)) 
                 AND (NC.CDPRODUTO    = (SELECT MAX(CDPRODUTO) FROM NDPCAL WHERE CHARINDEX(CDPRODUTO,PR.CDPRODUTO) = 1))
               UNION ALL
              /*receita por filial*/
              SELECT SUM(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRLO) AS PROTEINA,
                     SUM(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRLO * NC.VRFATCALNDPC) AS NPU
                FROM RECEFILI RF, NUTRPROD NP, PRODUTO PR, NDPCAL NC, PRODFILI
               WHERE (RF.CDFILIAL        = @P_CDFILIAL)                            
                 AND (RF.CDFILAUXPRAT    = @P_CDFILAUXPRAT)                        
                 AND (RF.CDPRATO         = @P_CDPRATO)                             
                 AND (RF.CDPRODUTO       = NP.CDPRODUTO)                           
                 AND (RF.CDPRODUTO       = PR.CDPRODUTO)                           
                 AND (PRODFILI.CDFILIAL  = @P_CDFILIAL)                            
                 AND (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                           
                 AND (NP.CDNUTRIENTE  = (SELECT MAX(CDPROTEINA) FROM PARAMGERAL))  
                 AND (NC.CDPRODUTO    = (SELECT MAX(CDPRODUTO) FROM NDPCAL WHERE CHARINDEX(CDPRODUTO,PR.CDPRODUTO) = 1))  
                 AND NOT EXISTS (SELECT CDPRATO
                                   FROM RECESERV
                                  WHERE (CDFILAUXPRAT = RF.CDFILAUXPRAT) 
                                    AND (CDPRATO      = RF.CDPRATO)      
                                    AND (CDFILIAL     = RF.CDFILIAL)     
                                    AND (CDSERVFILI   = @P_CDSERVFILI))          
               UNION ALL
              /*receita por tipo de cardpio*/
              SELECT SUM(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRPRTP) AS PROTEINA,
                     SUM(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRPRTP * NC.VRFATCALNDPC) AS NPU
                FROM RECETPCARD RT, NUTRPROD NP, PRODUTO PR, NDPCAL NC, PRODFILI
               WHERE (RT.CDFILCENTPME = @P_CDFILCENTPME)
                 AND (RT.CDTIPOCARD = @P_CDTIPOCARD)
                 AND (RT.CDFILAUXPRAT    = @P_CDFILAUXPRAT)                       
                 AND (RT.CDPRATO         = @P_CDPRATO)                            
                 AND (RT.CDPRODUTO       = NP.CDPRODUTO)                          
                 AND (RT.CDPRODUTO       = PR.CDPRODUTO)                          
                 AND (PRODFILI.CDFILIAL  = @P_CDFILIAL)                           
                 AND (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                          
                 AND (@P_MERENDA = 'S')
                 AND (NP.CDNUTRIENTE  = (SELECT MAX(CDPROTEINA) FROM PARAMGERAL)) 
                 AND (NC.CDPRODUTO    = (SELECT MAX(CDPRODUTO) FROM NDPCAL
                                          WHERE CHARINDEX(CDPRODUTO,PR.CDPRODUTO) = 1)) 
                 AND NOT EXISTS (SELECT CDPRATO
                                   FROM RECEFILI
                                  WHERE (CDFILAUXPRAT = RT.CDFILAUXPRAT) 
                                    AND (CDPRATO      = RT.CDPRATO)      
                                    AND (CDFILIAL     = @P_CDFILIAL)) 
                 AND NOT EXISTS (SELECT CDPRATO
                                   FROM RECESERV
                                  WHERE (CDFILAUXPRAT = RT.CDFILAUXPRAT) 
                                    AND (CDPRATO      = RT.CDPRATO)      
                                    AND (CDFILIAL     = @P_CDFILIAL)     
                                    AND (CDSERVFILI   = @P_CDSERVFILI))
               UNION ALL
              /*receita padro*/ 
              SELECT SUM(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRPR) AS PROTEINA,
                     SUM(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * QTCALCNUTRPR * NC.VRFATCALNDPC) AS NPU
                FROM RECEPADR RP, NUTRPROD NP, PRODUTO PR, NDPCAL NC, PRODFILI
               WHERE (RP.CDFILAUXPRAT    = @P_CDFILAUXPRAT)                       
                 AND (RP.CDPRATO         = @P_CDPRATO)                            
                 AND (RP.CDPRODUTO       = NP.CDPRODUTO)                          
                 AND (RP.CDPRODUTO       = PR.CDPRODUTO)                          
                 AND (PRODFILI.CDFILIAL  = @P_CDFILIAL)
                 AND (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                          
                 AND (NP.CDNUTRIENTE  = (SELECT MAX(CDPROTEINA) FROM PARAMGERAL)) 
                 AND (NC.CDPRODUTO    = (SELECT MAX(CDPRODUTO) FROM NDPCAL
                                          WHERE CHARINDEX(CDPRODUTO,PR.CDPRODUTO) = 1)) 
                 AND NOT EXISTS (SELECT CDPRATO
                                   FROM RECETPCARD
                                  WHERE (CDFILAUXPRAT = RP.CDFILAUXPRAT) 
                                    AND (CDPRATO      = RP.CDPRATO)      
                                    AND (CDFILCENTPME = @P_CDFILCENTPME)     
                                    AND (CDTIPOCARD  = @P_CDTIPOCARD)
                                    AND (@P_MERENDA = 'S'))
                 AND NOT EXISTS (SELECT CDPRATO
                                   FROM RECEFILI
                                  WHERE (CDFILAUXPRAT = RP.CDFILAUXPRAT) 
                                    AND (CDPRATO      = RP.CDPRATO)      
                                    AND (CDFILIAL     = @P_CDFILIAL)) 
                 AND NOT EXISTS (SELECT CDPRATO
                                   FROM RECESERV
                                  WHERE (CDFILAUXPRAT = RP.CDFILAUXPRAT) 
                                    AND (CDPRATO      = RP.CDPRATO)      
                                    AND (CDFILIAL     = @P_CDFILIAL)     
                                    AND (CDSERVFILI   = @P_CDSERVFILI))
        ) A
      ;

   OPEN CUR_PROTEINA;

   FETCH NEXT FROM CUR_PROTEINA INTO @V_CUR_PROTEINA, @V_NPU;

   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      FETCH NEXT FROM CUR_PROTEINA INTO @V_CUR_PROTEINA, @V_NPU;
   END;

   DEALLOCATE CUR_PROTEINA;

   IF @V_CALORIAS = 0 OR @V_CALORIAS IS NULL
   BEGIN
      SET @V_NDPCAL = 0;
   END ELSE BEGIN
      SET @V_NDPCAL = ISNULL(@V_CUR_PROTEINA,0) * ISNULL(@V_NPU,0) * 4 / @V_CALORIAS;
   END;
   
   RETURN(ROUND(@V_NDPCAL, 2));
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'NDPCAL_PRATO','0.00.000';
GO

